/*
 * ICFP 2002 Programming Contest
 * Entry by Seth Bruder
 *
 */

package sethbruder.icfp2002.entry;

import java.io.*;
import java.net.InetAddress;

import sethbruder.icfp2002.entry.player.Player;

public final class EntryMain {

	private static final boolean DEBUG_ON = false;

	public static void main(String[] args) {

		if (DEBUG_ON) Debug.activate();

		InetAddress serverHost = null;
		int serverPort = 0;

		//
		// Eval command line
		//

		try {
			if (args.length <= 0) {
				serverHost = InetAddress.getLocalHost();
                serverPort = 20005;
			}
			else if (args.length == 2) {
				serverHost = InetAddress.getByName(args[0]);
				serverPort = Integer.valueOf(args[1]).intValue();
				if (serverPort < 1)
					throw new Exception("Bad port number: " + serverPort);
			}
			else throw new Exception("Wrong number of arguments.");
		}
		catch (Exception e) {
			exit("Error processing command line arguments: "
				 + e.getMessage());
		}

		GameClient client;
		try {
			client = new GameClient(serverHost, serverPort);
		}
		catch (Exception e) {
			client = null;
			exit("Could not open connection to server: " + e);
		}

		InitInfo initInfo;
		try {
			initInfo = client.initialize();
		}
		catch (Exception e) {
			initInfo = null;
			e.printStackTrace();
			exit("Could not read initialization info from server: " + e);
		}

		Debug.print(initInfo.toString());

		final Player player = new Player(initInfo);
		Move move;
		TurnOutcome outcome;

		int skip = 0;
		BufferedReader stdin;
		if (DEBUG_ON)
			stdin = new BufferedReader(new InputStreamReader(System.in));

		try {
			while (true) {
				Debug.print(player);

				if (DEBUG_ON) {
					if (skip <= 0) {
						Debug.println("Enter number of steps to skip: ");
						String s = stdin.readLine();
						try {
							skip = Integer.valueOf(s).intValue();
						}
						catch (Exception e) {
							skip = 0;
						}
					}
					else skip--;
				}

				move = player.getMove();
				outcome = client.makeMove(move);

				Debug.print(outcome);

				player.assertTurnOutcome(move, outcome);
			}
		}
		catch (Exception e) {
			if (true) { //DEBUG_ON) {
				e.printStackTrace();
				System.out.println("Last lines read:\n"
								   + Debug.getLastLineRead());
				/*
				String s = null;
				try { 
					while (true) {
						s = client.readLine();
						if (s == null) break;
						System.out.println(s);
					}
				}
				catch (Exception e2) {}
				*/
			}
			exit("Done playing: " + e.getMessage());
		}
	}

	private static void exit(String s) {
		System.out.println(s);
		System.exit(1);
	}
}

