/*
 * ICFP 2002 Programming Contest
 * Entry by Seth Bruder
 *
 */

package sethbruder.icfp2002.entry;

import java.io.*;
import java.net.*;
import java.util.StringTokenizer;

final class GameClient {

	private final BufferedReader in;
	private final BufferedWriter out;

	public GameClient(InetAddress serverHost, int serverPort) 
		throws IOException {

		Debug.println("Connecting to: " + serverHost + " : " + serverPort);

		Socket socket = new Socket(serverHost, serverPort);
		in = new BufferedReader(new InputStreamReader(socket
													  .getInputStream()));
		out = new BufferedWriter(new OutputStreamWriter(socket
														.getOutputStream()));

		Debug.println("   ... connected.");
	}

	public InitInfo initialize() throws Exception {

		Debug.println("Initializing ...");

		InitInfo rv = new InitInfo();

		int width, height, x, y;
		String s, s2;
		StringTokenizer st;
		byte b;

		out.write("Player\n");
		out.flush();

		Debug.println("   sent \"Player\" string.");

		s = Debug.readLine(in);
		if (s == null) throw new Exception("Could not read line from server.");
		Debug.println("   read line: " + s);

		st = new StringTokenizer(s, " ", false);
		width = Integer.valueOf(st.nextToken()).intValue();
		height = Integer.valueOf(st.nextToken()).intValue();

		byte[][] board = new byte[width + 2][height + 2];
		for (y = 0; y <= (height + 1); y++) {
			if ((y > 0) && (y <= height)) {
				s = Debug.readLine(in);
				if (s == null)
					throw new Exception("Could not read line from server.");
			}

			for (x = 0; x <= (width + 1); x++) {
				if ((x == 0) || (x > width) || (y == 0) || (y > height))
					board[x][y] = Board.WALL;
				else {
					switch (s.charAt(x - 1)) {
						case '.':
							b = Board.EMPTY;
							break;
						case '~':
							b = Board.WATER;
							break;
						case '#':
							b = Board.WALL;
							break;
						case '@':
							b = Board.BASE;
							break;
						default:
							throw new Exception("Could not read board.");
					}
					board[x][y] = b;
				}
			}
		}

		rv.board = new Board(board);

		s = Debug.readLine(in);
		if (s == null)
			throw new Exception("Could not read line from server.");
		Debug.println("   read line: " + s);

		st = new StringTokenizer(s, " ", false);
		rv.playerId = Integer.valueOf(st.nextToken()).intValue();
		rv.capacity = Integer.valueOf(st.nextToken()).intValue();
		rv.cash = Integer.valueOf(st.nextToken()).intValue();

		s = Debug.readLine(in);
		s2 = Debug.readLine(in);
		if ((s == null) || (s2 == null))
			throw new Exception("Could not read line from server.");
		Debug.println("   read line: " + s);
		Debug.println("   read line: " + s2);

		rv.turnOutcome = new TurnOutcome(s, s2);

		Debug.println("   ... done initializing.\n");

		return rv;
	}

	public TurnOutcome makeMove(Move move) throws Exception {

		String s, s2;

		Debug.println("Making move: " + move.toString());
		out.write(move.toString() + "\n");
		out.flush();
		s = Debug.readLine(in);
		s2 = Debug.readLine(in);
		if ((s == null) || (s2 == null))
			throw new Exception("Could not read line from server.");

		//Debug.println("   read line: " + s);
		//Debug.println("   read line: " + s2);
		if (s.startsWith("Robot") && s.endsWith("died."))
			throw new Exception("Robot died: " + s2);

		return new TurnOutcome(s, s2);
	}

	/**
	 * For Debugging purposes only.  Called only by EntryMain.
	 **/
	public String readLine() throws Exception {
		return Debug.readLine(in);
	}
}
