/*
 * ICFP 2002 Programming Contest
 * Entry by Seth Bruder
 *
 */

package sethbruder.icfp2002.entry.player;

import java.util.HashSet;
import java.util.Set;
import java.util.HashMap;
import java.util.Map;

import sethbruder.icfp2002.entry.*;

final class PlayerState {

	public final int playerId;
	public int x;
	public int y;

	// Map from packageId (Integer) to PackageInfo
	public Set packages;

	public PlayerState(PlayerMoveOutcome pmo) {
		this.playerId = pmo.playerId;
		this.x = this.y = -1;
		this.packages = new HashSet();
		update(pmo);
	}

	public static Map getInitPlayerStates(InitInfo ii) {

		Map rv = new HashMap();

		PlayerMoveOutcome[] pmos
			= (PlayerMoveOutcome[])ii.turnOutcome.playerMoveOutcomes
			.values().toArray(new PlayerMoveOutcome[0]);

		PlayerState ps;
		for (int i = 0; i < pmos.length; i++) {
			ps = new PlayerState(pmos[i]);
			rv.put(new Integer(ps.playerId), ps);
		}

		return rv;
	}

	public void update(PlayerMoveOutcome pmo) {

		Move m;
		for (int i = 0; i < pmo.moves.length; i++) {
			m = pmo.moves[i];
			switch (m.type) {
				case Move.NORTH:
					y++;
					break;
				case Move.SOUTH:
					y--;
					break;
				case Move.EAST: 
					x++;
					break;
				case Move.WEST: 
					x--;
					break;
				case Move.PICK: 
					for (int j = 0; j < m.ids.length; j++)
						pick(m.ids[j]);
					break;
				case Move.DROP: 
					for (int j = 0; j < m.ids.length; j++)
						drop(m.ids[j]);
					break;
				case Move.POSITION:
					x = m.x;
					y = m.y;
					break;
				default:
					throw new RuntimeException();
			}
		}
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();

		sb.append("id=" + playerId + ", x=" + x +", y=" + y + ", pkgs=[");
		PackageInfo[] pis
			= (PackageInfo[])packages.toArray(new PackageInfo[0]);
		for (int i = 0; i < pis.length; i++) {
			if (i > 0) sb.append(", ");
			sb.append(pis[i].toString());
		}
		sb.append("]");
		return sb.toString();
	}

	private void pick(int id) {
		packages.add(PackageInfo.get(id));
	}

	private void drop(int id) {
		packages.remove(PackageInfo.get(id));
	}
}		
